@@ -1,11 +1,8 @@
-# Copyright (c) 2010-2011 Simplistix Ltd
-# See license.txt for license details.
-
 import atexit
 import warnings
 
 from zope.component import getSiteManager
-from zope.component.registry import Components
+from zope.interface.registry import Components
 
 
 class TestComponents:
@@ -16,6 +13,7 @@ class TestComponents:
     Instantiation will install an empty registry that will be returned
     by :func:`zope.component.getSiteManager`.
     """
+    __test__ = False
 
     instances = set()
     atexit_setup = False
