@@ -1,9 +1,12 @@
-from datetime import date
+from datetime import date, datetime
 
 from functools import partial
 
 from collections import namedtuple
-from mock import Mock, call
+
+from testfixtures.shouldraise import ShouldAssert
+from testfixtures.tests.sample1 import SampleClassA, SampleClassB, Slotted
+from testfixtures.mock import Mock, call
 from re import compile
 from testfixtures import (
     Comparison as C,
@@ -15,11 +18,11 @@ from testfixtures import (
     )
 from testfixtures.compat import (
     class_type_name, exception_module, PY3, xrange,
-    BytesLiteral, UnicodeLiteral
-    )
+    BytesLiteral, UnicodeLiteral,
+    PY2, PY_37_PLUS
+)
 from testfixtures.comparison import compare_sequence
 from unittest import TestCase
-from .compat import py_33_plus, py_2
 
 hexaddr = compile('0x[0-9A-Fa-f]+')
 
@@ -31,6 +34,7 @@ call_list_repr = repr(Mock().mock_calls.__class__)
 
 marker = object()
 
+_compare = compare
 
 class CompareHelper(object):
 
@@ -43,18 +47,20 @@ class CompareHelper(object):
         try:
             compare(*args, **kw)
         except Exception as e:
-            if not isinstance(e, AssertionError):
-                self.fail('Expected AssertionError, got %r' % e)
+            if not isinstance(e, AssertionError):  # pragma: no cover
+                raise
             actual = hexsub(e.args[0])
             if message is not None:
                 # handy for debugging, but can't be relied on for tests!
-                compare(actual, expected=message, show_whitespace=True)
-                self.assertEqual(actual, message)
+                _compare(actual, expected=message, show_whitespace=True)
+                assert actual == message
             else:
                 if not regex.match(actual):  # pragma: no cover
-                    self.fail('%r did not match %r' % (actual, regex.pattern))
+                    raise AssertionError(
+                        '%r did not match %r' % (actual, regex.pattern)
+                    )
         else:
-            self.fail('No exception raised!')
+            raise AssertionError('No exception raised!')
 
 
 class TestCompare(CompareHelper, TestCase):
@@ -64,12 +70,10 @@ class TestCompare(CompareHelper, TestCase):
         compare(o, o)
 
     def test_object_diff(self):
-        o1 = object()
-        o2 = object()
         self.check_raises(
-            o1, o2,
+            object(), object(),
             '<object object at ...> != <object object at ...>'
-            )
+        )
 
     def test_different_types(self):
         self.check_raises('x', 1, "'x' != 1")
@@ -88,7 +92,7 @@ class TestCompare(CompareHelper, TestCase):
         compare('x', 'x')
 
     def test_unicode_string_different(self):
-        if py_2:
+        if PY2:
             expected = "u'a' != 'b'"
         else:
             expected = "'a' != b'b'"
@@ -97,6 +101,43 @@ class TestCompare(CompareHelper, TestCase):
             expected
             )
 
+    def test_bytes_different(self):
+        if PY2:
+            expected = (
+                "\n" 
+                "'12345678901'\n"
+                '!=\n'
+                "'12345678902'"
+            )
+        else:
+            expected = (
+                "\n" 
+                "b'12345678901'\n"
+                '!=\n'
+                "b'12345678902'"
+            )
+        self.check_raises(
+            BytesLiteral('12345678901'),
+            BytesLiteral('12345678902'),
+            expected
+            )
+
+    def test_bytes_same_strict(self):
+        compare(actual=b'', expected=b'', strict=True)
+
+    if PY3:
+        def test_moar_bytes_different(self):
+            self.check_raises(
+                actual=b'{"byte_pound":"b\'\\\\xa3\'"}',
+                expected=b'{"byte_pound":"b\\\'\\xa3\'"}',
+                message = (
+                    "\n"
+                    "b'{\"byte_pound\":\"b\\\\\\'\\\\xa3\\\'\"}' (expected)\n"
+                    '!=\n'
+                    "b'{\"byte_pound\":\"b\\\'\\\\\\\\xa3\\\'\"}' (actual)"
+                )
+            )
+
     def test_string_diff_short(self):
         self.check_raises(
             '\n'+('x'*9), '\n'+('y'*9),
@@ -152,10 +193,7 @@ class TestCompare(CompareHelper, TestCase):
     def test_exception_different_object(self):
         e1 = ValueError('some message')
         e2 = ValueError('some message')
-        self.check_raises(
-            e1, e2,
-            "ValueError('some message',) != ValueError('some message',)"
-            )
+        compare(e1, e2)
 
     def test_exception_different_object_c_wrapper(self):
         e1 = ValueError('some message')
@@ -165,10 +203,16 @@ class TestCompare(CompareHelper, TestCase):
     def test_exception_diff(self):
         e1 = ValueError('some message')
         e2 = ValueError('some other message')
-        self.check_raises(
-            e1, e2,
-            "ValueError('some message',) != ValueError('some other message',)"
-            )
+        if PY_37_PLUS:
+            self.check_raises(
+                e1, e2,
+                "ValueError('some message') != ValueError('some other message')"
+                )
+        else:
+            self.check_raises(
+                e1, e2,
+                "ValueError('some message',) != ValueError('some other message',)"
+                )
 
     def test_exception_diff_c_wrapper(self):
         e1 = ValueError('some message')
@@ -176,12 +220,14 @@ class TestCompare(CompareHelper, TestCase):
         self.check_raises(
             C(e1), e2,
             ("\n"
-             "  <C(failed):{0}.ValueError>\n"
-             "  args:('some message',) != ('some other message',)\n"
-             "  </C>"
-             " != "
-             "ValueError('some other message',)"
-             ).format(exception_module))
+             "<C(failed):{}.ValueError>\n"
+             "attributes differ:\n"
+             "'args': ('some message',) (Comparison) "
+             "!= ('some other message',) (actual)\n"
+             "</C>"
+             " != ValueError('some other message'{})"
+             ).format(exception_module,
+                      '' if PY_37_PLUS else ','))
 
     def test_sequence_long(self):
         self.check_raises(
@@ -376,13 +422,18 @@ class TestCompare(CompareHelper, TestCase):
             )
 
     def test_dict_consistent_ordering_types_same(self):
+        if PY3:
+            same = "[6, None]\n"
+        else:
+            same = "[None, 6]\n"
+
         self.check_raises(
             {None: 1, 6: 2, 1: 3},
             {None: 1, 6: 2, 1: 4},
             "dict not as expected:\n"
-            "\n"
-            'same:\n'
-            "[6, None]\n"
+            "\n"+
+            'same:\n'+
+            same+
             "\n"
             "values differ:\n"
             "1: 3 != 4"
@@ -651,7 +702,7 @@ class TestCompare(CompareHelper, TestCase):
         compare((1, 2, 3), [1, 2, 3])
 
     def test_tuple_and_list_strict(self):
-        if py_2:
+        if PY2:
             expected = ("(1, 2, 3) (<type 'tuple'>) != "
                         "[1, 2, 3] (<type 'list'>)")
         else:
@@ -792,14 +843,14 @@ b
             self.assertEqual(y, {2: 2})
             self.assertEqual(context.get_option('foo'), 'bar')
             return 'not equal'
-        with ShouldRaise(AssertionError('not equal')):
+        with ShouldAssert('not equal'):
             compare({1: 1}, {2: 2},
                     foo='bar',
                     comparers={dict: compare_dict})
 
     def test_register_more_specific(self):
         class_ = namedtuple('Test', 'x')
-        with ShouldRaise(AssertionError('compare class_')):
+        with ShouldAssert('compare class_'):
             compare(class_(1), class_(2),
                     comparers={
                     tuple: Mock(return_value='compare tuple'),
@@ -838,15 +889,15 @@ b
                 )
 
     def test_list_subclass(self):
-        m = Mock()
-        m.aCall()
-        # Mock().method_calls is a list subclass
+        class  MyList(list): pass
+        a_list = MyList([1])
+        b_list = MyList([2])
         self.check_raises(
-            [call.bCall()], m.method_calls,
+            a_list, b_list,
             "sequence not as expected:\n\n"
             "same:\n[]\n\n"
-            "first:\n[call.bCall()]\n\n"
-            "second:\n[call.aCall()]"
+            "first:\n[1]\n\n"
+            "second:\n[2]"
             )
 
     def test_strict_okay(self):
@@ -1212,7 +1263,7 @@ b
             )
 
     def test_strict_nested_different(self):
-        if py_2:
+        if PY2:
             expected = "[1, 2] (<type 'list'>) != (1, 3) (<type 'tuple'>)"
         else:
             expected = "[1, 2] (<class 'list'>) != (1, 3) (<class 'tuple'>)"
@@ -1296,7 +1347,7 @@ b
 
     def test_invalid_zero_args(self):
         with ShouldRaise(TypeError(
-                'Exactly two objects needed, you supplied: []'
+                'Exactly two objects needed, you supplied:'
         )):
             compare()
 
@@ -1312,6 +1363,12 @@ b
         )):
             compare('x', 'y', 'z')
 
+    def test_invalid_because_of_typo(self):
+        with ShouldRaise(TypeError(
+                "Exactly two objects needed, you supplied: ['x'] {'expceted': 'z'}"
+        )):
+            compare('x', expceted='z')
+
     def test_dont_raise(self):
         self.assertEqual(compare('x', 'y', raises=False), "'x' != 'y'")
 
@@ -1331,32 +1388,32 @@ b
             yield self.OrmObj(1)
             yield self.OrmObj(2)
 
-        def compare_orm_obj(x, y, context):
-            if x.a != y.a:
-                return 'OrmObj: %s != %s' % (x.a, y.a)
-
         self.check_raises(
             message=(
-                "sequence not as expected:\n\n"
-                "same:\n(OrmObj: 1,)\n\n"
-                "expected:\n(OrmObj: 3,)\n\n"
-                "actual:\n(OrmObj: 2,)\n\n"
-                "While comparing [1]: OrmObj: 3 != 2"
+                "sequence not as expected:\n"
+                "\n"
+                "same:\n"
+                "(OrmObj: 1,)\n"
+                "\n"
+                "expected:\n"
+                "(OrmObj: 3,)\n"
+                "\n"
+                "actual:\n"
+                "(OrmObj: 2,)\n"
+                '\n'
+                'While comparing [1]: OrmObj not as expected:\n'
+                '\n'
+                'attributes differ:\n'
+                "'a': 3 (expected) != 2 (actual)"
             ),
             expected=[self.OrmObj(1), self.OrmObj(3)],
             actual=query_set(),
-            comparers={self.OrmObj: compare_orm_obj},
             ignore_eq=True
         )
 
     def test_django_orm_is_horrible_part_2(self):
 
-        def compare_orm_obj(x, y, context):
-            return context.different(x.a, y.a, '.a')
-
-        t_compare = partial(compare,
-                            comparers={self.OrmObj: compare_orm_obj},
-                            ignore_eq=True)
+        t_compare = partial(compare, ignore_eq=True)
 
         t_compare(self.OrmObj(1), self.OrmObj(1))
         t_compare(self.OrmObj('some longish string'),
@@ -1365,10 +1422,7 @@ b
                   self.OrmObj(date(2016, 1, 1)))
 
     def test_django_orm_is_horrible_part_3(self):
-        self.check_raises(
-            message=(
-                "OrmObj: 1 (expected) != OrmObj: 1 (actual)"
-            ),
+        compare(
             expected=self.OrmObj(1),
             actual=self.OrmObj(1),
             ignore_eq=True
@@ -1381,3 +1435,340 @@ b
             actual=2,
             ignore_eq=True
         )
+
+    def test_mock_call_same(self):
+        m = Mock()
+        m.foo(1, 2, x=3)
+        compare(m.mock_calls, m.mock_calls)
+
+    def test_mock_call_same_strict(self):
+        m = Mock()
+        m.foo(1, 2, x=3)
+        compare(m.mock_calls, m.mock_calls, strict=True)
+
+    def test_calls_different(self):
+        m1 =Mock()
+        m2 =Mock()
+        m1.foo(1, 2, x=3, y=4)
+        m2.bar(1, 3, x=7, y=4)
+
+        self.check_raises(
+            m1.mock_calls,
+            m2.mock_calls,
+            "sequence not as expected:\n"
+            "\n"
+            "same:\n"
+            "[]\n"
+            "\n"
+            "first:\n"
+            "[call.foo(1, 2, x=3, y=4)]\n"
+            "\n"
+            "second:\n"
+            "[call.bar(1, 3, x=7, y=4)]"
+            "\n\n"
+            'While comparing [0]: \n'
+            "'call.foo(1, 2, x=3, y=4)'\n"
+            '!=\n'
+            "'call.bar(1, 3, x=7, y=4)'"
+        )
+
+    def test_compare_arbitrary_nested_same(self):
+        compare(SampleClassA([SampleClassB()]),
+                SampleClassA([SampleClassB()]))
+
+    def test_compare_different_vars(self):
+        obj1 = SampleClassB(1)
+        obj1.same = 42
+        obj1.foo = '1'
+        obj2 = SampleClassB(2)
+        obj2.same = 42
+        obj2.bar = '2'
+        self.check_raises(
+            obj1, obj2,
+            "SampleClassB not as expected:\n"
+            "\n"
+            "attributes same:\n"
+            "['same']\n"
+            "\n"
+            'attributes in first but not second:\n'
+            "'foo': '1'\n"
+            "\n"
+            'attributes in second but not first:\n'
+            "'bar': '2'\n"
+            '\n'
+            'attributes differ:\n'
+            "'args': (1,) != (2,)\n"
+            '\n'
+            "While comparing .args: sequence not as expected:\n"
+            '\n'
+            'same:\n'
+            '()\n'
+            '\n'
+            'first:\n'
+            '(1,)\n'
+            '\n'
+            'second:\n'
+            '(2,)'
+        )
+
+    def test_compare_arbitrary_nested_diff(self):
+        class OurClass:
+            def __init__(self, *args):
+                self.args = args
+            def __repr__(self):
+                return '<OurClass obj>'
+        self.check_raises(
+            OurClass(OurClass(1)),
+            OurClass(OurClass(2)),
+            "OurClass not as expected:\n"
+            "\n"
+            'attributes differ:\n'
+            "'args': (<OurClass obj>,) != (<OurClass obj>,)\n"
+            '\n'
+            'While comparing .args: sequence not as expected:\n'
+            '\n'
+            'same:\n'
+            '()\n'
+            '\n'
+            'first:\n'
+            '(<OurClass obj>,)\n'
+            '\n'
+            'second:\n'
+            '(<OurClass obj>,)\n'
+            '\n'
+            'While comparing .args[0]: OurClass not as expected:\n'
+            '\n'
+            'attributes differ:\n'
+            "'args': (1,) != (2,)\n"
+            '\n'
+            'While comparing .args[0].args: sequence not as expected:\n'
+            '\n'
+            'same:\n'
+            '()\n'
+            '\n'
+            'first:\n'
+            '(1,)\n'
+            '\n'
+            'second:\n'
+            '(2,)'
+        )
+
+    def test_compare_slotted_same(self):
+        compare(Slotted(1, 2), Slotted(1, 2))
+
+    def test_compare_slotted_diff(self):
+        self.check_raises(
+            Slotted(1, 2),
+            Slotted(1, 3),
+            "Slotted not as expected:\n"
+            "\n"
+            "attributes same:\n"
+            "['x']\n"
+            "\n"
+            'attributes differ:\n'
+            "'y': 2 != 3"
+        )
+
+    def test_empty_sets(self):
+        compare(set(), set())
+
+    def test_empty_sets_strict(self):
+        compare(set(), set(), strict=True)
+
+    def test_datetime_not_equal(self):
+        self.check_raises(
+            datetime(2001, 1, 1),
+            datetime(2001, 1, 2),
+            "datetime.datetime(2001, 1, 1, 0, 0) != "
+            "datetime.datetime(2001, 1, 2, 0, 0)"
+        )
+
+    def test_inherited_slots(self):
+
+        class Parent(object):
+            __slots__ = ('a',)
+
+        class Child(Parent):
+            __slots__ = ('b',)
+
+            def __init__(self, a, b):
+                self.a, self.b = a, b
+
+        self.check_raises(
+            Child(1, 'x'),
+            Child(2, 'x'),
+            'Child not as expected:\n'
+            '\n'
+            'attributes same:\n'
+            "['b']\n"
+            '\n'
+            'attributes differ:\n'
+            "'a': 1 != 2"
+        )
+
+    def test_empty_child_slots(self):
+
+        class Parent(object):
+            __slots__ = ('a',)
+
+            def __init__(self, a):
+                self.a = a
+
+        class Child(Parent):
+            __slots__ = ()
+
+        compare(Child(1), Child(1))
+
+    def test_partial_callable_different(self):
+
+        def foo(x): pass
+        def bar(y): pass
+
+        self.check_raises(
+            partial(foo),
+            partial(bar),
+            (
+                'partial not as expected:\n'
+                '\n'
+                'attributes same:\n'
+                "['args', 'keywords']\n"
+                '\n'
+                'attributes differ:\n'
+                "'func': {foo} != {bar}\n"
+                '\n'
+                'While comparing .func: {foo} != {bar}'
+            ).format(foo=hexsub(repr(foo)), bar=hexsub(repr(bar))))
+
+    def test_partial_args_different(self):
+
+        def foo(x): pass
+
+        self.check_raises(
+            partial(foo, 1),
+            partial(foo, 2),
+            'partial not as expected:\n'
+            '\n'
+            'attributes same:\n'
+            "['func', 'keywords']\n"
+            '\n'
+            'attributes differ:\n'
+            "'args': (1,) != (2,)\n"
+            '\n'
+            'While comparing .args: sequence not as expected:\n'
+            '\n'
+            'same:\n'
+            '()\n'
+            '\n'
+            'first:\n'
+            '(1,)\n'
+            '\n'
+            'second:\n'
+            '(2,)'
+        )
+
+    def test_partial_kw_different(self):
+
+        def foo(x): pass
+
+        self.check_raises(
+            partial(foo, x=1, y=3),
+            partial(foo, x=2, z=4),
+            'partial not as expected:\n'
+            '\n'
+            'attributes same:\n'
+            "['args', 'func']\n"
+            '\n'
+            'attributes differ:\n'
+            "'keywords': {'x': 1, 'y': 3} != {'x': 2, 'z': 4}\n"
+            '\n'
+            'While comparing .keywords: dict not as expected:\n'
+            '\n'
+            'in first but not second:\n'
+            "'y': 3\n"
+            '\n'
+            'in second but not first:\n'
+            "'z': 4\n"
+            '\n'
+            'values differ:\n'
+            "'x': 1 != 2"
+        )
+
+    def test_partial_equal(self):
+
+        def foo(x): pass
+
+        compare(partial(foo, 1, x=2), partial(foo, 1, x=2))
+
+    def test_repr_and_attributes_equal(self):
+
+        class Wut(object):
+            def __repr__(self):
+                return 'Wut'
+            def __eq__(self, other):
+                return False
+
+        self.check_raises(
+            Wut(),
+            Wut(),
+            "Both x and y appear as 'Wut', but are not equal!"
+        )
+
+        self.check_raises(
+            expected=Wut(),
+            actual=Wut(),
+            message="Both expected and actual appear as 'Wut', but are not equal!"
+        )
+
+    def test_not_recursive(self):
+        self.check_raises(
+            {1: 'foo', 2: 'foo'},
+            {1: 'bar', 2: 'bar'},
+            "dict not as expected:\n"
+            "\n"
+            "values differ:\n"
+            "1: 'foo' != 'bar'\n"
+            "2: 'foo' != 'bar'\n"
+            "\n"
+            "While comparing [1]: 'foo' != 'bar'"
+            "\n\n"
+            "While comparing [2]: 'foo' != 'bar'"
+            )
+
+
+class TestIgnore(CompareHelper):
+
+    class Parent(object):
+        def __init__(self, id, other):
+            self.id = id
+            self.other = other
+        def __repr__(self):
+            return '<{}:{}>'.format(type(self).__name__, self.id)
+
+    class Child(Parent): pass
+
+    def test_ignore_attributes(self):
+        compare(self.Parent(1, 3), self.Parent(2, 3), ignore_attributes={'id'})
+
+    def test_ignore_attributes_different_types(self):
+        self.check_raises(
+            self.Parent(1, 3),
+            self.Child(2, 3),
+            '<Parent:1> != <Child:2>',
+            ignore_attributes={'id'}
+        )
+
+    def test_ignore_attributes_per_type(self):
+        ignore = {self.Parent: {'id'}}
+        compare(self.Parent(1, 3), self.Parent(2, 3), ignore_attributes=ignore)
+        self.check_raises(
+            self.Child(1, 3),
+            self.Child(2, 3),
+            'Child not as expected:\n'
+            '\n'
+            'attributes same:\n'
+            "['other']\n"
+            '\n'
+            'attributes differ:\n'
+            "'id': 1 != 2",
+            ignore_attributes=ignore
+        )
